#!/bin/bash

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
# OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# This is a sample bash script that creates a customized FutureGrid image. 
# For any questions, please refer to:
# http://hpc-sched.cs.sfu.ca/


# creating an initial CentOS image:
ssh blagodurov@india.futuregrid.org
module load futuregrid
fg-shell -u blagodurov
use image
generate -o centos -v 6 -a x86_64 -s wget,emacs,python26 -g
#The image is located in .//blagodurov2945785520.tgz
#The image and the manifest generated are packaged in a tgz file.

# copying locally onto my machine to install necessary software:
# CentOS 5:
scp blagodurov@india.futuregrid.org:/N/u/blagodurov/blagodurov2945785520.tgz .
# CentOS 6:
scp blagodurov@india.futuregrid.org:/N/u/blagodurov/blagodurov1196976793.tgz .
tar -xzf blagodurov2945785520.tgz

# mounting:
mkdir fgmnt
mount -o loop blagodurov2945785520.img fgmnt/
# make sure that only image itself is mounted, no dev and proc:
umount /mnt/disk1/fg/fgmnt/proc
umount -l /mnt/disk1/fg/fgmnt/dev/

# changing size of the image:
# checking image filesystem:
mount
dd if=/dev/zero of=new_image.img bs=1 count=1 seek=5G
chmod 777 new_image.img
mkfs.ext3 new_image.img
mkdir fgmnt2
mount -o loop new_image.img fgmnt2/
cp -prf ./fgmnt/* ./fgmnt2/
umount fgmnt
umount fgmnt2
mv blagodurov2945785520.img blagodurov2945785520.old.img
mv new_image.img blagodurov2945785520.img

# mounting again:
mount -o loop blagodurov1196976793.img fgmnt/
# adding temporary networking support for chroot:
mount -t proc /proc/ /cs/ckpts/fg/fgmnt/proc/
mount --rbind /dev/ /cs/ckpts/fg/fgmnt/dev/

# logging in:
cd fgmnt/
sudo chroot .
# check if ping google.com works, if not then launch dhclient

# properly setting up history inside the image to log everything I do. Making sure that bash is set up properly:
more /root/.bash_profile
  shopt -s histappend
  export HISTFILE=/root/.history
  PROMPT_COMMAND="history -a;history -n;$PROMPT_COMMAND"
  HISTTIMEFORMAT='%F %T '
  declare -x HISTFILESIZE=1000000000
  declare -x HISTSIZE=1000000
more /root/.bash
  .bash_logout   .bash_profile  .bashrc
  bash-3.2# more /root/.bashrc
  if [ -f ~/.bash_profile ]; then
     source ~/.bash_profile
  fi
# then change HOME and execute bash one more time:
export HOME=/root/
bash



# Installing software:
chkconfig --level 3456 sshd on
chkconfig --level 0123456 iptables off
chkconfig --level 0123456 ip6tables off

yum check-update
yum install rpm-build ncurses-devel ncurses redhat-rpm-config patchutils hmaccalc rng-tools gcc make screen nano telnet OpenIPMI OpenIPMI-tools

# installing OpenVZ as is described in:
# http://wiki.openvz.org/Quick_installation
# http://wiki.centos.org/HowTos/Virtualization/OpenVZ
cd /etc/yum.repos.d
wget http://download.openvz.org/openvz.repo
rpm --import  http://download.openvz.org/RPM-GPG-Key-OpenVZ
# installing kernel:
# http://wiki.openvz.org/Kernel_build
cd /root/
wget http://download.openvz.org/kernel/branches/rhel6-2.6.32/current/vzkernel-2.6.32-042stab049.6.src.rpm
TOPDIR=$(rpm --eval "%{_topdir}"); echo $TOPDIR
mkdir $TOPDIR
rpm -ihv /root/vzkernel-2.6.32-042stab049.6.src.rpm
cd $TOPDIR/SPECS
rngd -r /dev/urandom
rpmbuild -bp kernel.spec
cd $TOPDIR/BUILD/kernel-2.6.32-220.4.1.el6/linux-2.6.32-042stab049.6/
cp /root/rpmbuild/SOURCES/config-vz .config
make menuconfig
cp .config configs/kernel-2.6.32-x86_64.config
rm -f configs/kernel-2.6.32-x86_64-debug.config
cd $TOPDIR/SPECS
rpmbuild -ba kernel.spec
yum install kernel-firmware virtual-vzkernel-install
cd /root/rpmbuild/RPMS/x86_64/
rpm -ihv --replacefiles --force vzkernel-2.6.32-042stab049.6.x86_64.rpm
rpm -ihv --replacefiles --force vzkernel-devel-2.6.32-042stab049.6.x86_64.rpm
rpm -ihv --replacefiles --force vzkernel-headers-2.6.32-042stab049.6.x86_64.rpm 


# checking grub:
vi /boot/grub/grub.conf
# title Cent OS 6 OpenVZ
# root (hd0,0)
# kernel /boot/vmlinuz-2.6.32-042stab049.6 root=/dev/sda1
# module /boot/initramfs-2.6.32-042stab049.6.img

echo "CheckHostIP no" >> /root/.ssh/config
echo "StrictHostKeyChecking no" >> /root/.ssh/config
vzsplit -n 1 -f max-limits

# change hostname manually, if necessary:
vi /etc/sysconfig/network
# HOSTNAME=testhost



# include password auth in sshd (if necessary):
vi /etc/ssh/sshd_config
/etc/init.d/sshd restart

# merge vi /etc/sysctl.conf as described in http://wiki.openvz.org/Quick_installation:
# On Hardware Node we generally need
# packet forwarding enabled and proxy arp disabled
net.ipv4.ip_forward = 1
net.ipv6.conf.default.forwarding = 1
net.ipv6.conf.all.forwarding = 1
net.ipv4.conf.default.proxy_arp = 0
# Enables source route verification
net.ipv4.conf.all.rp_filter = 1
# Enables the magic-sysrq key
kernel.sysrq = 1
# We do not want all our interfaces to send redirects
net.ipv4.conf.default.send_redirects = 1
net.ipv4.conf.all.send_redirects = 0

# increase open file limit as is specified in http://www.cyberciti.biz/faq/linux-unix-nginx-too-many-open-files/ :
vi /etc/sysctl.conf
# Append/modify the following line:
# fs.file-max = 1000000
vi /etc/security/limits.conf
# # Set soft and hard limit for all users or nginx user as follows:
# *       soft    nofile   100000
# *       hard    nofile   300000

# reboot or reload the changes with sysctl command:
sysctl -p

# to see the current limits:
ulimit -Hn
ulimit -Sn

vi /usr/sbin/vzmigrate
# blowfish -> arcfour
# tmpdir="/var/tmp" -> tmpdir="/vz/dump"
# setting max limits to prevent some weird errors:
vzsplit -n 1 -f max-limits

# reboot


# copying OpenVZ container data (OpenMPI and SPEC MPI 2007 pre-installed within):
# example of conf file (locked pages should be 0 to perform online migration!):
cat /etc/vz/conf/104.conf
KMEMSIZE="unlimited"
DCACHESIZE="unlimited"
LOCKEDPAGES="0"
PRIVVMPAGES="unlimited"
SHMPAGES="unlimited"
NUMPROC="unlimited"
VMGUARPAGES="0:unlimited"
OOMGUARPAGES="0:unlimited"
NUMTCPSOCK="unlimited"
NUMFLOCK="unlimited"
NUMPTY="unlimited"
NUMSIGINFO="unlimited"
TCPSNDBUF="unlimited"
TCPRCVBUF="unlimited"
OTHERSOCKBUF="unlimited"
DGRAMRCVBUF="unlimited"
NUMOTHERSOCK="unlimited"
NUMFILE="unlimited"
NUMIPTENT="unlimited"
PHYSPAGES="0:unlimited"
SWAPPAGES="0:unlimited"
AVNUMPROC="unlimited"

# Disk quota parameters (in form of softlimit:hardlimit)
DISKSPACE="29982254:32980480"
DISKINODES="2162661:2378928"
QUOTATIME="0"

# CPU fair scheduler parameter
CPUUNITS="1000"
VE_ROOT="/vz/root/$VEID"
VE_PRIVATE="/vz/private/$VEID"
OSTEMPLATE="centos-6-x86_64"
ORIGIN_SAMPLE="vswap-256m"
IP_ADDRESS="192.168.196.240"
NAMESERVER="192.168.196.3"
FEATURES="nfs:on "
HOSTNAME="bernal"
CAPABILITY="CHOWN:on DAC_OVERRIDE:on DAC_READ_SEARCH:on FOWNER:on FSETID:on KILL:on SETGID:on SETUID:on SETPCAP:on LINUX_IMMUTABLE:on NET_BIND_SERVICE:on NET_BROADCAST:on NET_ADMIN:on NET_RAW:on IPC_LOCK:on IPC_OWNER:on SYS_MODULE:on SYS_RAWIO:on SYS_CHROOT:on SYS_PTRACE:on SYS_PACCT:on SYS_ADMIN:on SYS_BOOT:on SYS_NICE:on SYS_RESOURCE:on SYS_TIME:on SYS_TTY_CONFIG:on MKNOD:on LEASE:on VE_ADMIN:on FS_MASK:on "


cp 102.conf /etc/vz/conf/
cp -rf 102 /vz/private/

# OpenVZ traffic accounting (part I):
# http://wiki.openvz.org/Traffic_accounting_with_iptables
# adding rules:
cd /home/storage2/
cat > container-ips.txt
192.168.196.238
192.168.196.239
192.168.196.240
192.168.196.241
192.168.196.242
192.168.196.243
192.168.196.244
192.168.196.245
192.168.196.246
192.168.196.247
192.168.196.248
192.168.196.249
192.168.196.250
192.168.196.251
192.168.196.235
192.168.196.236


# Installing netem for emulating network disruptions:
# description: http://www.linuxfoundation.org/collaborate/workgroups/networking/netem
modprobe sch_netem
rmmod sch_netem

# merge vi /etc/sysctl.conf as described in http://fasterdata.es.net/fasterdata/host-tuning/linux/:
# increase TCP max buffer size setable using setsockopt()
# 16 MB with a few parallel streams is recommended for most 10G paths
# 32 MB might be needed for some very long end-to-end 10G or 40G paths
net.core.rmem_max = 16777216 
net.core.wmem_max = 16777216 
# increase Linux autotuning TCP buffer limits 
# min, default, and max number of bytes to use
# (only change the 3rd value, and make it 16 MB or more)
net.ipv4.tcp_rmem = 4096 87380 16777216
net.ipv4.tcp_wmem = 4096 65536 16777216
# recommended to increase this for 10G NICS
net.core.netdev_max_backlog = 30000
# these should be the default, but just to be sure
net.ipv4.tcp_timestamps = 1
net.ipv4.tcp_sack = 1

# using:
tc qdisc add dev eth0 root netem delay 100ms
tc qdisc change dev eth0 root netem delay 100ms 10ms
tc qdisc change dev eth0 root netem delay 100ms 10ms 25%
tc qdisc change dev eth0 root netem loss 0.1%
# be sure to execute the following from the console, otherwise you will loose connection:
tc qdisc change dev eth0 root netem loss 80.1%

#removing netem rules:
tc qdisc del dev eth0 root netem


# Installing cluster framework (Torque PBS + Maui)
chmod -R 777 /home/storage2/newmpi/

# 1. On the head node:
# install PBS packages:
# check if these files are in /etc/yum.repos.d/
# epel.repo  epel-testing.repo
# issue yum repolist
# if these repos are not working, change https to http in them
cd /etc/pki/rpm-gpg/
wget http://ftp.riken.jp/Linux/fedora/epel/RPM-GPG-KEY-EPEL-6
yum install torque-server torque-client torque-devel torque-mom torque-scheduler torque-docs

# install munge
# description why: http://www.eu-emi.eu/products/-/asset_publisher/z2MT/content/torque-server-config
yum install munge
/usr/sbin/create-munge-key
chown -R munge:munge /etc/munge/
chown -R munge:munge /var/log/munge/
chown -R munge:munge /var/run/munge/
chown -R munge:munge /var/lib/munge/
chown munge:munge /var/log/munge/munged.log*
chkconfig munge on
/etc/init.d/munge start

#Installing Maui:
cd /home/sba70/clusters/maui-3.3
./configure
# remove any optimization options (i.e. -O1, -O2 or -O3) from generated Makefile, otherwise there will be problem with sprintf! - see http://www.clusterresources.com/pipermail/mauiusers/2011-February/004551.html
gmake
gmake install
# Replace localhost in the field SERVERHOST of /usr/local/maui/maui.cfg with the fully qualified domain name of the machine. 
# Ensure the field ADMIN1 of /usr/local/maui/maui.cfg is "root". 

# add
# export PATH=${PATH}:/usr/local/maui/bin/:/usr/local/maui/sbin/
# export ROOTPATH=${ROOTPATH}:/usr/local/maui/bin/:/usr/local/maui/sbin/
# to the end of:
# /etc/bash/bashrc
# or
# /etc/bashrc 

#add: pathmunge /usr/lib64/openmpi/bin/
#to:
# /etc/bashrc
# /etc/profile

# the files are in /var/lib/torque
# change /etc/torque/server_name

# create a new config db for pbs server
chmod -Rf 755 /var 
chmod -Rf 777 /var/spool/torque/spool/
chmod +t /var/spool/torque/spool/
chmod -Rf 777 /var/spool/torque/undelivered/
chmod +t /var/spool/torque/undelivered/

chmod -Rf 755 /var 
chmod -Rf 777 /var/lib/torque/spool/
chmod +t /var/lib/torque/spool/
chmod -Rf 777 /var/lib/torque/undelivered/
chmod +t /var/lib/torque/undelivered/

pbs_server -t create
cd /usr/share/doc/torque-2.1.10/
sudo ./torque.setup root
# make sure the polling of moms is infrequent, otherwise the moms may be marked as down during online migration 
# (and subsequently their jobs will be terminated)
qmgr -c 'set server node_check_rate = 1800'
qmgr -c 'set server node_ping_rate = 1800'
qmgr -c 'set server tcp_timeout = 1800'
# check the configuration for pbs server:
qmgr -c 'p s'

# in /etc/torque/pbs_environment, append to PATH the contents of "echo $PATH"
# on server add /var/lib/torque/server_priv/nodes file:
cat > /var/lib/torque/server_priv/nodes
deco np=12
marina np=12
bernal np=12
castro np=12
chinatown np=12
cole np=12
wharf np=12
haight np=12
hayes np=12
sunset np=12
tenderloin np=12
soma np=12
potrero np=12
rushill np=12
union np=12
pacific np=12

# create a non root user for job submission:
useradd sergey -G wheel
passwd sergey
# copy .ssh from root
chown -R sergey /home/sergey/.ssh/
chgrp -R sergey /home/sergey/.ssh/
chmod -R 600 /home/sergey/.ssh/
chmod go-w $HOME /home/sergey/.ssh/
# check that you can login into sergey without password
cd /home/sergey/
tar -pczf ssh.tar.gz .ssh

chmod -R 777 /home/storage2/newmpi/


# starting:
maui
/etc/init.d/pbs_server start
# stopping:
/etc/init.d/pbs_server stop
schedctl -k


# 2. On compute nodes:
scp headnode:/etc/yum.repos.d/epel* /etc/yum.repos.d/
cd /etc/pki/rpm-gpg/
wget http://ftp.riken.jp/Linux/fedora/epel/RPM-GPG-KEY-EPEL-6
yum install torque-client torque-mom munge
# the files are in /var/lib/torque
# change /etc/torque/server_name
# append to PATH the contents of echo $PATH in /etc/torque/pbs_environment
# change /etc/torque/mom/config:
$pbsserver headnode
$logevent 0x1fff
$status_update_time 1800
$check_poll_time 600
$node_check_interval 60
$timeout 600

scp headnode:/etc/torque/server_name /etc/torque/server_name
scp headnode:/etc/torque/mom/config /etc/torque/mom/config
scp headnode:/etc/torque/pbs_environment /etc/torque/pbs_environment
scp headnode:/etc/munge/munge.key /etc/munge/munge.key

chown munge:munge /etc/munge/munge.key
chkconfig munge on
/etc/init.d/munge start
chown munge:munge /var/log/munge/munged.log*

# starting:
chkconfig pbs_mom on
/etc/init.d/pbs_mom start

# recompile pbs_mom without locked pages (otherwise online migration will not work). 
# Make sure that the version of mom is the same as the one installed with yum ("pbs_mom -v").
/etc/init.d/pbs_mom stop
cp /home/storage2/torque-2.5.7.tar.gz .
tar -xzf torque-2.5.7.tar.gz
cd torque-2.5.7
./configure --prefix=/usr --build=x86_64-pc-linux-gnu --host=x86_64-pc-linux-gnu --mandir=/usr/share/man --infodir=/usr/share/info --datadir=/usr/share --sysconfdir=/etc --localstatedir=/var/lib --libdir=/usr/lib64 --enable-syslog --disable-drmaa --enable-high-availability --with-server-home=/var/lib/torque --with-environ=/etc/torque/pbs_environment --disable-gcc-warnings --disable-cpusets --with-rcp=scp --enable-unixsockets=no --disable-posixmemlock --enable-plock-daemons=0 --enable-munge-auth --disable-server --disable-clients
make
make install
/etc/init.d/pbs_mom start

# create a non root user for job submission:
useradd sergey -G wheel
passwd sergey
# copy ssh.tar.gz from headnode:
cd /home/sergey/
scp headnode:/home/sergey/ssh.tar.gz /home/sergey/
tar -xzf ssh.tar.gz


# stopping:
/etc/init.d/pbs_mom stop

# OpenVZ traffic accounting (part II). On every compute node:
for i in `cat /home/storage2/container-ips.txt`;  do for j in `cat /home/storage2/container-ips.txt`;  do iptables -A FORWARD -d $i -s $j; done; done
# checking:
iptables -nvx -L FORWARD
iptables -Z

# starrting all the necessary daemons upon boot:
# add: /home/storage2/hpc-upon-boot.sh
# to: vi /etc/rc.local


# installing Clavis user level daemon:
# on CentOS:
yum install numactl-devel gcc-c++ ncurses-devel libpcap-devel elfutils-devel newt-devel binutils-devel zlib-static
# on Ubuntu:
# apt-get install libnuma-dev elfutils libnewt-dev binutils-dev zlib-bin libelf-dev libnuma1 numactl iotop
# place -lm and so on after the c files in scheduler_script!

# installing perf:
cd /root/
tar -xzf linux-2.6.34.9.tar.gz
cd linux-2.6.34.9
cd tools/perf/
mkdir /usr/libexec/perf-core
HOME="/usr"
make
make install


# Installing OpenIPMI and checking power consumption:
# on gentoo:
# check that IMPI is enabled in kernel
# emerge -up sys-libs/openipmi sys-apps/ipmitool
/etc/init.d/ipmi start
ipmitool sdr type 'Current'

# Finishing touches: 
# unmounting:
exit
exit
cd ..
# killing dhclient:
kill 18636
umount /mnt/disk1/fg/fgmnt/proc
umount -l /mnt/disk1/fg/fgmnt/dev/
umount /mnt/disk1/fg/fgmnt


# Checking the gerenrated FG image by booting its contents in KVM:
# creating an initial KVM image file (perform only once):
dd if=/dev/zero of=new_image2.img bs=1 count=1 seek=32G
# to be able to boot it in KVM, you need to make the created file a *bootable* image.
# For that, boot the KVM with, say, CentOS 6 LiveCD and install default OS into it (perform only once).

# then, mount the image file:
emerge -u sys-fs/multipath-tools
losetup /dev/loop1 cluster-image.img
kpartx -a /dev/loop1
mount /dev/mapper/loop1p1 fgmnt2

# backup the /boot directory, erase everything, copy all from FG image, add the /boot contents back:  
cd /mnt/disk1/fg/
cd fgmnt2/
cp -pr boot/ ../from_image/
rm -rf *
mount -o loop blagodurov1196976793.img fgmnt/
umount /mnt/disk1/fg/fgmnt/proc
umount -l /mnt/disk1/fg/fgmnt/dev/
cp -pr ../fgmnt/* .
rm -rf boot/
cp -pr ../from_image/boot/ .
umount fgmnt2/
# if you need to test kernel as well, make sure to backup the contents of fgmnt2 first, then, carefully, overwrite it with the new kernel.
# then boot 
virsh create ./fg0.xml


#uploading the resultuing image onto FG:
cd /mnt/disk1/fg/
tar -pczf blagodurov1196976793-upd2.tgz blagodurov1196976793.img blagodurov1196976793.manifest.xml
scp /mnt/disk1/fg/blagodurov1196976793-upd2.tgz blagodurov@india.futuregrid.org:/N/u/blagodurov/



# Installing MPICH2:
# download latest src.rpm from http://www.mcs.anl.gov/research/projects/mpich2/downloads/index.php?s=downloads
# install version 1.3 or higher as they have Hydra by default which is much better and easy to use in comparison with mpd:
#wget http://kojipkgs.fedoraproject.org/packages/mpich2/1.4.1p1/1.el5/src/mpich2-1.4.1p1-1.el5.src.rpm
rpm -ivh mpich2-1.4.1p1-1.el5.src.rpm
TOPDIR=$(rpm --eval "%{_topdir}"); echo $TOPDIR
cd /root/rpmbuild/SPECS/
rpmbuild -bp mpich2.spec
rpmbuild -ba mpich2.spec
rpm -ihv --replacefiles --force mpich2-*
cp /root/rpmbuild/RPMS/x86_64/mpich2-* /home/storage2/
# then just istall thus compiled packages on all the nodes/containers with:
cd /home/storage2/
yum install mpich2-*

#add: pathmunge /usr/lib64/mpich2/bin/ to:
/etc/bashrc # add before "unset pathmunge" string
/etc/profile # add inside if operator adter "Path manipulation"
# create a file mpich2.conf with "/usr/lib64/mpich2/lib/" in /etc/ld.so.conf.d/

# Switching back to OpenMPI:
sed -i 's/mpich2/openmpi/g' /etc/profile
sed -i 's/mpich2/openmpi/g' /etc/bashrc
sed -i 's/mpich2/openmpi/g' /etc/ld.so.conf.d/mpich2.conf
ldconfig
# comment out:
scp headnode:/etc/profile.d/mpich2-x86_64.* /etc/profile.d/
vi /etc/profile.d/mpich2-x86_64.sh
vi /etc/profile.d/mpich2-x86_64.csh

