#!/bin/bash

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
# OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# This is a sample bash script that deploys the contention-aware HPC framework 
# on the FG nodes booted with the customized image.
# It depends on the following scripts that can be also found in this directory:
# hpc_keys.sh - sets up the password-less login between the framework nodes
# nfs-storage-contents.sh - mounts NFS, deploys OpenVZ containers on the nodes and 
# brings the framework up and running. nfs-storage-contents.sh in turn depends on:
#    103.sh - configures each OpenVZ container
#    nfs-mount-client.sh - mounts NFS clients
#    hpc-upon-boot.sh - launches the framework on every node

# For any questions, please refer to:
# http://hpc-sched.cs.sfu.ca/


# login and book the nodes:
ssh -i /cs/systems/home/sba70/.ssh/id_rsa blagodurov@india.futuregrid.org
qsub -I -l os=centosblagodurov1196976793 -l nodes=2

# temporary remembering the passphrase (if any):
ssh-agent bash
ssh-add

qstat -n <jobid>
# Check what nodes I can really login into. If there are some more nodes than what is in the output of the above command, 
# check if containers are running there from the last time, if so, destroy them.
date > /N/u/blagodurov/ini_output
LAST_NODE_ID=111
for i in `seq 1 $LAST_NODE_ID`
do
   CUR_OUTPUT=$(ssh root@i$i "vzlist" 2>&1)
   if  [[ "$CUR_OUTPUT" =~ "closed" ]] || [[ "$CUR_OUTPUT" =~ "denied" ]] || [[ "$CUR_OUTPUT" =~ "No route" ]];
   then
      :
   else   
      echo i$i
      echo i$i >> /N/u/blagodurov/ini_output
      echo $CUR_OUTPUT >> /N/u/blagodurov/ini_output      
   fi
done
nano /N/u/blagodurov/ini_output

# Put all the nodes except one from "qstat -n <jobid>" output into the file nfs-storage-contents/nodes.txt.
# The one last node is used as a head node for the framework (e.g. i56).

hn=i56
export hn
# uploading root keys:
scp ./hpc_keys/* root@$hn:/root/.ssh/
ssh root@$hn ". ./.ssh/hpc_keys.sh"
for i in `cat /N/u/blagodurov/nfs-storage-contents/nodes.txt`;  do
  CUR_NODE=$i;

  scp ./hpc_keys/* root@$CUR_NODE:/root/.ssh/
  ssh root@$CUR_NODE ". ./.ssh/hpc_keys.sh"
done

# creating and uploading the deployment tarball
tar -pczf nfs-storage-contents.tar.gz nfs-storage-contents
scp nfs-storage-contents* root@$hn:/root/
# begin deployment
date > /N/u/blagodurov/deploy_output
ssh root@$hn "./nfs-storage-contents.sh" >> /N/u/blagodurov/deploy_output 2>> /N/u/blagodurov/deploy_output

