#!/bin/bash

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
# INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
# PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
# HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
# CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
# OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

# This is a sample bash script that mounts NFS, deploys OpenVZ containers on the nodes and 
# brings the framework up and running.
# script name: ./nfs-storage-contents.sh

# For any questions, please refer to:
# http://hpc-sched.cs.sfu.ca/


qterm -t quick
/usr/local/maui/bin/schedctl -k
rm -f /var/lib/torque/server_priv/serverdb

umount -l /home/storage2/
rm -rf /home/storage*
/etc/init.d/nfs stop
sed -i '/storage2/d' /etc/fstab

# creating nfs server:
mkdir /home/storage
mkdir /home/storage2
sed -i '/storage/d' /etc/exports
echo '/home/storage/ *(rw,fsid=1,sync,subtree_check,no_root_squash)' >> /etc/exports
HEAD_NODE=`hostname`
echo $HEAD_NODE':/home/storage/    /home/storage2/         nfs     auto,rw,soft,nfsvers=3,rsize=32768,wsize=32768,intr,noatime  0 0' >> /etc/fstab
sed -i 's/#RPCNFSDCOUNT=8/RPCNFSDCOUNT=64/g' /etc/sysconfig/nfs
/etc/init.d/nfs start
mount /home/storage2/

# extracting contents:
tar -xzf /root/nfs-storage-contents.tar.gz -C /home/storage2/
cd /home/storage2/
mv nfs-storage-contents/* .
rm -f /root/nfs-storage-contents.tar.gz
rmdir nfs-storage-contents/

# extracting workload:
tar -xzf new-mpi.tar.gz
rm -f new-mpi.tar.gz

chmod -R 777 /home/storage2/scripts/

# deploying containers:
CUR_CTID=102
for i in `cat /home/storage2/nodes.txt`;  do 
  CUR_NODE=$i;
  
  ssh $CUR_NODE "/etc/init.d/vz stop"
  ssh $CUR_NODE "rm -rf /vz/private/1*"
  ssh $CUR_NODE "rm -rf /vz/root/1*"

  # mounting nfs client:
  ssh $CUR_NODE "rm -f /root/nfs-mount-client.sh"
  scp /home/storage2/nfs-mount-client.sh $CUR_NODE:/root/
  ssh $CUR_NODE "/root/nfs-mount-client.sh $HEAD_NODE"

  # copying containers:
  ssh $CUR_NODE "cp -f /home/storage2/vz.conf /etc/vz/vz.conf"
  ssh $CUR_NODE "cp /home/storage2/103* /root/"

  ssh $CUR_NODE "tar -xpzf 103.tar.gz -C /root/"
  ssh $CUR_NODE "/root/103.sh $CUR_CTID $HEAD_NODE"
  CUR_CTID=`expr $CUR_CTID + 1`
  ssh $CUR_NODE "/root/103.sh $CUR_CTID $HEAD_NODE"
  CUR_CTID=`expr $CUR_CTID + 1`
  ssh $CUR_NODE "rm -fr /root/103*"
done

# deploying infrastructure:

yum -y install iotop

#continuing with the keys:
cd /root/
. ./.ssh/sergey_hpc_keys.sh

echo "" >> /etc/hosts
echo "" >> /etc/hosts
echo "# OpenVZ container internal IPs for blagodurov HPC framework" >> /etc/hosts
echo "" >> /etc/hosts
cat /home/storage2/hosts >> /etc/hosts
echo "" >> /etc/hosts
echo "# end OpenVZ container internal IPs for blagodurov HPC framework" >> /etc/hosts
echo "" >> /etc/hosts

/home/storage2/hpc-upon-boot.sh

CUR_CTID=102
for i in `cat /home/storage2/nodes.txt`;  do 
  CUR_NODE=$i;

  ssh $CUR_NODE "yum -y install iotop"
  scp /etc/hosts $CUR_NODE:/etc/hosts

  ssh $CUR_NODE "cp /etc/hosts /vz/private/$CUR_CTID/etc/hosts"
  CUR_CTID=`expr $CUR_CTID + 1`
  ssh $CUR_NODE "cp /etc/hosts /vz/private/$CUR_CTID/etc/hosts"
  CUR_CTID=`expr $CUR_CTID + 1`

  ssh $CUR_NODE "/home/storage2/hpc-upon-boot.sh"
done

echo $HEAD_NODE > /var/lib/torque/server_name
echo $HEAD_NODE > /var/spool/torque/server_name
cp /home/storage2/nodes /var/lib/torque/server_priv/nodes

chmod -Rf 755 /var 
chmod -Rf 777 /var/spool/torque/spool/
chmod +t /var/spool/torque/spool/
chmod -Rf 777 /var/spool/torque/undelivered/
chmod +t /var/spool/torque/undelivered/

chmod -Rf 755 /var 
chmod -Rf 777 /var/lib/torque/spool/
chmod +t /var/lib/torque/spool/
chmod -Rf 777 /var/lib/torque/undelivered/
chmod +t /var/lib/torque/undelivered/

pbs_server -t create
# make sure the polling of moms is infrequent, otherwise the moms may be marked as down during online migration 
# (and subsequently their jobs will be terminated)
qmgr -c 'set server node_check_rate = 1800'
qmgr -c 'set server node_ping_rate = 1800'
qmgr -c 'set server tcp_timeout = 1800'

qmgr -c 'create queue batch'
qmgr -c 'set queue batch queue_type = Execution'
qmgr -c 'set queue batch resources_default.nodes = 1'
qmgr -c 'set queue batch resources_default.walltime = 01:00:00'
qmgr -c 'set queue batch enabled = True'
qmgr -c 'set queue batch started = True'

qmgr -c 'set server scheduling = True'
qmgr -c "set server managers = root@$HEAD_NODE"
qmgr -c "set server operators = root@$HEAD_NODE"
qmgr -c 'set server default_queue = batch'
qmgr -c 'set server mom_job_sync = True'
qmgr -c 'set server keep_completed = 90000'

# check the configuration for pbs server:
qmgr -c 'p s'

chmod 777 /var/run/munge/munge.socket.2

sed -i 's/SERVERHOST            miga/SERVERHOST            '$HEAD_NODE'/g' /usr/local/maui/maui.cfg
/usr/local/maui/sbin/maui


